﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using System.Data;
using System.Text.RegularExpressions;

namespace Xenon.ListGraphic
{

    /// <summary>
    /// function の行一覧
    /// </summary>
    public class FunctionTable
    {

        /// <summary>
        /// local の行
        /// </summary>
        private DataTable Table
        {
            get;
            set;
        }


        public FunctionTable()
        {
            this.Table = new DataTable();
            this.Table.Columns.Add("Program");
            this.Table.Columns.Add("Line", Type.GetType("System.Int32"));
            this.Table.Columns.Add("Name");
            this.Table.Columns.Add("Params");
        }


        public DataRow[] Select(String where, String orderBy)
        {
            return this.Table.Select(where, orderBy);
        }

        public DataRow NewRow()
        {
            return this.Table.NewRow();
        }

        public DataRowCollection Rows
        {
            get
            {
                return this.Table.Rows;
            }
        }

        public void Add(DataRow row)
        {
            this.Table.Rows.Add(row);
        }


        public void ScanFileLines_1syume(string file, string[] lines)
        {
            //------------------------------
            //
            // 変換   ＜サンプル＞
            //
            // function gra_playerChr_walk( chrSymbol, graName )
            // ↓
            // gra_playerChr_walk
            // chrSymbol, graName
            //
            //------------------------------
            Regex r = new Regex(
                @"\s*"+//隙間
                @"function\s*"+//＜サンプル＞ function
                @"([^\s\(]*)\s*" +//＜サンプル＞ gra_playerChr_walk
                @"\(\s*"+       //＜サンプル＞ (
                @"([^\)]*)" +     //＜サンプル＞ chrSymbol, graName
                @"\)\s*",       //＜サンプル＞ )
                RegexOptions.Compiled | RegexOptions.Singleline);

            int lineCount = 1;
            foreach (string line in lines)
            {
                Match m = r.Match(line);
                if (m.Success)
                {
                    //------------------------------------------------------------
                    // local の行
                    //------------------------------------------------------------
                    DataRow row = this.NewRow();
                    row["Program"] = file;
                    row["Line"] = lineCount;
                    row["Name"] = m.Groups[1];
                    row["Params"] = m.Groups[2];
                    this.Add(row);
                    goto gt_EndParse;
                }

            gt_EndParse:
                lineCount++;
            }
        }


        /// <summary>
        /// report_local.csv
        /// </summary>
        /// <returns></returns>
        public string ReportCsv()
        {
            StringBuilder sb = new StringBuilder();

            sb.Append("No");
            sb.Append(",Program");
            sb.Append(",Line");
            sb.Append(",Name");
            sb.Append(",Params");
            sb.AppendLine();

            int no = 1;
            foreach (DataRow row in this.Rows)
            {
                sb.Append(Actions_Csv.EscapeCell(no.ToString()));
                sb.Append(",");
                sb.Append(Actions_Csv.EscapeCell(row["Program"].ToString()));
                sb.Append(",");
                sb.Append(Actions_Csv.EscapeCell(row["Line"].ToString()));
                sb.Append(",");
                sb.Append(Actions_Csv.EscapeCell(row["Name"].ToString()));
                sb.Append(",");
                sb.Append(Actions_Csv.EscapeCell(row["Params"].ToString()));
                sb.AppendLine();

                no++;
            }

            return sb.ToString();
        }


        /// <summary>
        /// ソースを遡ってfunction署名を調べます。
        /// </summary>
        /// <param name="program1"></param>
        /// <param name="name2"></param>
        /// <param name="line1"></param>
        /// <returns></returns>
        public DataRow FindFunctionSignature(string program1, int line1)
        {
            DataRow localRow = null;

            StringBuilder sb = new StringBuilder();

            sb.Append("Program like '");
            sb.Append(program1);
            sb.Append("' and ");

            sb.Append("Line<");
            sb.Append(line1);

            DataRow[] localRows = this.Select(sb.ToString(), "Line");

            if (0 < localRows.Length)
            {
                localRow = localRows[localRows.Length - 1];
            }

            return localRow;
        }

    }

}
